/*
 * xmms-mad - mp3 plugin for xmms
 * Copyright (C) 2001-2002 Sam Clegg - See COPYING
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This file was contributed in whole be Samuel Krempp <krempp@crans.org>
 *
 * $Id: xmms-mad.c,v 1.5 2005/03/06 03:04:38 sam Exp $
 */
#include "xmms-mad.h"
#include "stdlib.h"
#include "math.h"
#include "ctype.h"

struct APETagFooterStruct {
    unsigned char   ID       [8];
    unsigned char   Version  [4];
    unsigned char   Length   [4];
    unsigned char   TagCount [4];
    unsigned char   Flags    [4];
    unsigned char   Reserved [8];
};

unsigned long Read_LE_Uint32 ( const char* p )
{
    return ((unsigned long)p[0] <<  0) |
           ((unsigned long)p[1] <<  8) |
           ((unsigned long)p[2] << 16) |
           ((unsigned long)p[3] << 24);
}

int uncase_strcmp(const char* s1, const char* s2) {
  int l1 = strlen(s1);
  int l2 = strlen(s2);
  int i;
  for(i=0; i< l1 && i<l2; ++i) {
    if(toupper(s1[i]) < toupper(s2[i]))
      return -1;
  }
  if(l1==l2) 
    return 0;
  return (l1<l2) ? -1 : +1;
}

gdouble strgain2double(gchar *s, int len) {
  gdouble res = g_strtod(s, NULL); // gain, in dB.
  if(res==0)
    return 1;
  return pow(10, res/20);
}

// Reads APE v2.0 tag ending at current pos in fp

int ReadAPE2Tag ( FILE* fp, struct mad_info_t * file_info)
{
    unsigned long               vsize;
    unsigned long               isize;
    unsigned long               flags;
    char*              buff;
    char*              p;
    char*              end;
    struct APETagFooterStruct   T;
    unsigned long               TagLen;
    unsigned long               TagCount;

    if ( fseek ( fp, -sizeof T, SEEK_CUR ) != 0 )
        return 18;
    if ( fread ( &T, 1, sizeof T, fp ) != sizeof T )
        return 2;
    if ( memcmp ( T.ID, "APETAGEX", sizeof T.ID ) != 0 )
        return 3;
    if ( Read_LE_Uint32 ((char*)T.Version) != 2000 )
        return 4;
    TagLen = Read_LE_Uint32 ((char*)T.Length);
    if ( TagLen < sizeof T )
        return 5;
    if ( fseek ( fp, - TagLen, SEEK_CUR ) != 0 )
        return 6;
    if ( (buff = (char *)malloc ( TagLen )) == NULL ) {
        return 7;
    }
    if ( fread ( buff, 1, TagLen - sizeof T, fp ) != TagLen - sizeof T ) {
        free ( buff );
        return 8;
    }

    TagCount = Read_LE_Uint32 ((char*)T.TagCount);
    end = buff + TagLen - sizeof (T);
    for ( p = buff; p < end && TagCount--; ) {
        vsize = Read_LE_Uint32 ( p ); p += 4;
        flags = Read_LE_Uint32 ( p ); p += 4;
        isize = strlen ((char *)p);

        if ( isize > 0 && vsize > 0 ) {
          gdouble * scale = NULL;
          gchar* * str = NULL;
          if(uncase_strcmp(p, "REPLAYGAIN_ALBUM_GAIN") ==0 ) {
            scale = & file_info->replaygain_album_scale;
            str = & file_info->replaygain_album_str;
          }
          if(uncase_strcmp(p, "REPLAYGAIN_TRACK_GAIN") ==0) {
            scale = & file_info->replaygain_track_scale;
            str = & file_info->replaygain_track_str;
          }
          if(str != NULL) {
            assert(scale != NULL);
            *scale = strgain2double(p+isize+1, vsize);
            *str = g_strndup(p+isize+1, vsize);
          }

          //* case of peak info tags : */
          str = NULL;
          if(uncase_strcmp(p, "REPLAYGAIN_TRACK_PEAK") ==0) {
            scale = & file_info->replaygain_track_peak;
            str = & file_info->replaygain_track_peak_str;
          }
          if(uncase_strcmp(p, "REPLAYGAIN_ALBUM_PEAK") ==0) {
            scale = & file_info->replaygain_album_peak;
            str = & file_info->replaygain_album_peak_str;
          }
          if(str != NULL) {
            *scale = g_strtod(p+isize+1, NULL);
            *str = g_strndup(p+isize+1, vsize);
          }

          /* mp3gain additional tags : 
             the gain tag translates to scale = 2^(gain/4), 
             i.e., in dB : 20*log(2)/log(10)*gain/4
             -> 1.501*gain dB
          */
          if(uncase_strcmp(p, "MP3GAIN_UNDO") ==0) {
            str = & file_info->mp3gain_undo_str;
            scale = & file_info->mp3gain_undo;
            assert(4<vsize); /* this tag is +left,+right */
            *str = g_strndup(p+isize+1, vsize);
            *scale = 1.50515 * atoi(*str); 
          }
          if(uncase_strcmp(p, "MP3GAIN_MINMAX") ==0) {
            str = & file_info->mp3gain_minmax_str;
            scale = & file_info->mp3gain_minmax;
            *str = g_strndup(p+isize+1, vsize);
            assert(4<vsize); /* this tag is min,max */
            *scale = 1.50515 * (atoi((*str)+4) - atoi(*str));
          }
        }
        p += isize + 1 + vsize;
    }

    free ( buff );

    return 0;
}

int find_offset(FILE* fp) {
  static const char* key = "APETAGEX";
  char buff[20000];
  int N=0;
  if ( fseek ( fp, -20000, SEEK_CUR ) != 0 )
    ;
  if ( (N=fread ( buff, 1, 20000, fp )) < 16) 
    return 1;
  int matched=0;
  int i,last_match=-1;
  for(i=0; i<N; ++i) {
    if(buff[i]== key[matched])
      ++matched;
    else {
      if(matched==5 && buff[i]=='P')
        matched=2; // got "APET" + "AP"
      else
        matched=0;
    }
    if(matched==8) {
      last_match=i;
      matched=0;
    }
  }
  if(last_match==-1)
    return 1;
  return last_match+1-8 +sizeof(struct APETagFooterStruct)-N;
}  

void input_read_replaygain(struct mad_info_t* file_info) {
  file_info->has_replaygain = FALSE;
  file_info->replaygain_album_scale = -1;
  file_info->replaygain_track_scale = -1;
  file_info->mp3gain_undo = -77;
  file_info->mp3gain_minmax = -77;


    FILE*   fp;

    if ( (fp = fopen ( file_info->filename, "rb" )) == NULL )
        return;

    if ( fseek ( fp, 0L, SEEK_END ) != 0 ) {
        fclose (fp);
        return ;
    }
    long pos = ftell (fp);
    int res = -1;
    int try = 0;
    while(res!=0 && try < 10) {
      // try skipping an id3 tag
      fseek(fp, pos, SEEK_SET);
      fseek(fp, try * -128, SEEK_CUR);
      res = ReadAPE2Tag( fp, file_info);
      ++try;
    }
    if(res!=0) {
      // try brute search (don't want to parse all possible kinds of tags..)
      fseek(fp, pos, SEEK_SET);
      int offs = find_offset(fp);
      if(offs<=0) { // found !
        fseek(fp, pos, SEEK_SET);
        fseek(fp, offs, SEEK_CUR);
        res = ReadAPE2Tag( fp, file_info);
        if(res != 0) {
          g_message("hmpf, was supposed to find a tag.. offs=%d, res=%d",
                    offs, res);
        }
      }
    }
      

#ifdef DEBUG
    if(res==0) { // got APE tags, show the result
      printf("RG album scale= %g, RG track scale = %g, in %s \n\n", 
             file_info->replaygain_album_scale,
             file_info->replaygain_track_scale,
             file_info->filename);
    }
#endif

    if(file_info->replaygain_album_scale != -1 || file_info->replaygain_track_scale  != -1)
      file_info -> has_replaygain = TRUE;

    fclose (fp);
}
